@@ -15,8 +15,6 @@
  * Doc/Manual/SWIGPlus.html for details.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_allocate_cxx[] = "$Id: allocate.cxx 13936 2012-11-24 14:15:20Z wsfulton $";
-
 #include "swigmod.h"
 #include "cparse.h"
 
@@ -423,7 +421,7 @@ class Allocate:public Dispatcher {
 		while (cc) {
 		  Node *cp = cc;
 		  if (classname) {
-		    Setattr(cp, "classname", classname);
+		    Setattr(cp, "extendsmartclassname", classname);
 		  }
 		  Setattr(cp, "allocate:smartpointeraccess", "1");
 		  /* If constant, we have to be careful */
@@ -561,7 +559,11 @@ Allocate():
   virtual int classDeclaration(Node *n) {
     Symtab *symtab = Swig_symbol_current();
     Swig_symbol_setscope(Getattr(n, "symtab"));
-
+    save_value<Node*> oldInclass(inclass);
+    save_value<AccessMode> oldAcessMode(cplus_mode);
+    save_value<int> oldExtendMode(extendmode);
+    if (Getattr(n, "template"))
+      extendmode = 0;
     if (!CPlusPlus) {
       /* Always have default constructors/destructors in C */
       Setattr(n, "allocate:default_constructor", "1");
@@ -582,7 +584,6 @@ Allocate():
 	}
       }
     }
-
     inclass = n;
     String *kind = Getattr(n, "kind");
     if (Strcmp(kind, "class") == 0) {
@@ -661,7 +662,7 @@ Allocate():
     }
 
     if (!Getattr(n, "allocate:has_destructor")) {
-      /* No destructor was defined.  We need to check a few things here too */
+      /* No destructor was defined */
       List *bases = Getattr(n, "allbases");
       int allows_destruct = 1;
 
@@ -678,13 +679,13 @@ Allocate():
     }
 
     if (!Getattr(n, "allocate:has_assign")) {
-      /* No destructor was defined.  We need to check a few things here too */
+      /* No assignment operator was defined */
       List *bases = Getattr(n, "allbases");
       int allows_assign = 1;
 
       for (int i = 0; i < Len(bases); i++) {
 	Node *n = Getitem(bases, i);
-	/* If base class does not allow default destructor, we don't allow it either */
+	/* If base class does not allow assignment, we don't allow it either */
 	if (Getattr(n, "allocate:has_assign")) {
 	  allows_assign = !Getattr(n, "allocate:noassign");
 	}
@@ -695,13 +696,13 @@ Allocate():
     }
 
     if (!Getattr(n, "allocate:has_new")) {
-      /* No destructor was defined.  We need to check a few things here too */
+      /* No new operator was defined */
       List *bases = Getattr(n, "allbases");
       int allows_new = 1;
 
       for (int i = 0; i < Len(bases); i++) {
 	Node *n = Getitem(bases, i);
-	/* If base class does not allow default destructor, we don't allow it either */
+	/* If base class does not allow new operator, we don't allow it either */
 	if (Getattr(n, "allocate:has_new")) {
 	  allows_new = !Getattr(n, "allocate:nonew");
 	}
@@ -730,7 +731,6 @@ Allocate():
 
     /* Only care about default behavior.  Remove temporary values */
     Setattr(n, "allocate:visit", "1");
-    inclass = 0;
     Swig_symbol_setscope(symtab);
     return SWIG_OK;
   }
@@ -773,7 +773,7 @@ Allocate():
       /* Check to see if this is a static member or not.  If so, we add an attribute
          cplus:staticbase that saves the current class */
 
-      if (checkAttribute(n, "storage", "static")) {
+      if (Swig_storage_isstatic(n)) {
 	Setattr(n, "cplus:staticbase", inclass);
       }
 
@@ -781,18 +781,26 @@ Allocate():
       if (cplus_mode != PUBLIC) {
 	if (Strcmp(name, "operator =") == 0) {
 	  /* Look for a private assignment operator */
-	  Setattr(inclass, "allocate:has_assign", "1");
+	  if (!GetFlag(n, "deleted"))
+	    Setattr(inclass, "allocate:has_assign", "1");
 	  Setattr(inclass, "allocate:noassign", "1");
 	} else if (Strcmp(name, "operator new") == 0) {
 	  /* Look for a private new operator */
-	  Setattr(inclass, "allocate:has_new", "1");
+	  if (!GetFlag(n, "deleted"))
+	    Setattr(inclass, "allocate:has_new", "1");
 	  Setattr(inclass, "allocate:nonew", "1");
 	}
       } else {
 	if (Strcmp(name, "operator =") == 0) {
-	  Setattr(inclass, "allocate:has_assign", "1");
+	  if (!GetFlag(n, "deleted"))
+	    Setattr(inclass, "allocate:has_assign", "1");
+	  else
+	    Setattr(inclass, "allocate:noassign", "1");
 	} else if (Strcmp(name, "operator new") == 0) {
-	  Setattr(inclass, "allocate:has_new", "1");
+	  if (!GetFlag(n, "deleted"))
+	    Setattr(inclass, "allocate:has_new", "1");
+	  else
+	    Setattr(inclass, "allocate:nonew", "1");
 	}
 	/* Look for smart pointer operator */
 	if ((Strcmp(name, "operator ->") == 0) && (!GetFlag(n, "feature:ignore"))) {
@@ -819,7 +827,7 @@ Allocate():
 		  }
 		  List *methods = smart_pointer_methods(sc, 0, isconst);
 		  Setattr(inclass, "allocate:smartpointer", methods);
-		  Setattr(inclass, "allocate:smartpointerbase", base);
+		  Setattr(inclass, "allocate:smartpointerpointeeclassname", Getattr(sc, "name"));
 		} else {
 		  /* Hmmm.  The return value is not a pointer.  If the type is a value
 		     or reference.  We're going to chase it to see if another operator->()
