@@ -35,6 +35,7 @@ struct Scanner {
   String *error;                /* Last error message (if any) */
   int     error_line;           /* Error line number */
   int     freeze_line;          /* Suspend line number updates */
+  List   *brackets;             /* Current level of < > brackets on each level */
 };
 
 typedef struct Locator {
@@ -44,6 +45,9 @@ typedef struct Locator {
 } Locator;
 static int follow_locators = 0;
 
+void Scanner_push_brackets(Scanner*);
+void Scanner_clear_brackets(Scanner*);
+
 /* -----------------------------------------------------------------------------
  * NewScanner()
  *
@@ -65,6 +69,8 @@ Scanner *NewScanner(void) {
   s->error = 0;
   s->error_line = 0;
   s->freeze_line = 0;
+  s->brackets = NewList();
+  Scanner_push_brackets(s);
   return s;
 }
 
@@ -77,12 +83,13 @@ Scanner *NewScanner(void) {
 void DelScanner(Scanner * s) {
   assert(s);
   Delete(s->scanobjs);
+  Delete(s->brackets);
   Delete(s->text);
   Delete(s->file);
   Delete(s->error);
   Delete(s->str);
   free(s->idstart);
-  free(s);
+  free(s); 
 }
 
 /* -----------------------------------------------------------------------------
@@ -96,6 +103,7 @@ void Scanner_clear(Scanner * s) {
   Delete(s->str);
   Clear(s->text);
   Clear(s->scanobjs);
+  Scanner_clear_brackets(s);
   Delete(s->error);
   s->str = 0;
   s->error = 0;
@@ -260,6 +268,74 @@ static void freeze_line(Scanner *s, int val) {
   s->freeze_line = val;
 }
 
+/* -----------------------------------------------------------------------------
+ * Scanner_brackets()
+ *
+ * Returns the number of brackets at the current depth.
+ * ----------------------------------------------------------------------------- */
+int*
+Scanner_brackets(Scanner *s) {
+  return (int*)(**((void***)Getitem(s->brackets, 0))); /* TODO: Use VoidObj*->ptr instead of void** */
+}
+
+/* -----------------------------------------------------------------------------
+ * Scanner_clear_brackets()
+ *
+ * Resets the current depth and clears all brackets.
+ * Usually called at the end of statements;
+ * ----------------------------------------------------------------------------- */
+void
+Scanner_clear_brackets(Scanner *s) {
+  Clear(s->brackets);
+  Scanner_push_brackets(s); /* base bracket count should always be created */
+}
+
+/* -----------------------------------------------------------------------------
+ * Scanner_inc_brackets()
+ *
+ * Increases the number of brackets at the current depth.
+ * Usually called when '<' was found.
+ * ----------------------------------------------------------------------------- */
+void
+Scanner_inc_brackets(Scanner *s) {
+  (*Scanner_brackets(s))++;
+}
+
+/* -----------------------------------------------------------------------------
+ * Scanner_dec_brackets()
+ *
+ * Decreases the number of brackets at the current depth.
+ * Usually called when '>' was found.
+ * ----------------------------------------------------------------------------- */
+void
+Scanner_dec_brackets(Scanner *s) {
+  (*Scanner_brackets(s))--;
+}
+
+/* -----------------------------------------------------------------------------
+ * Scanner_push_brackets()
+ *
+ * Increases the depth of brackets.
+ * Usually called when '(' was found.
+ * ----------------------------------------------------------------------------- */
+void
+Scanner_push_brackets(Scanner *s) {
+  int *newInt = malloc(sizeof(int));
+  *newInt = 0;
+  Push(s->brackets, NewVoid(newInt, free));
+}
+
+/* -----------------------------------------------------------------------------
+ * Scanner_pop_brackets()
+ *
+ * Decreases the depth of brackets.
+ * Usually called when ')' was found.
+ * ----------------------------------------------------------------------------- */
+void
+Scanner_pop_brackets(Scanner *s) {
+  Delitem(s->brackets, 0);
+}
+
 /* -----------------------------------------------------------------------------
  * retract()
  *
@@ -407,13 +483,15 @@ static void get_escape(Scanner *s) {
  * ----------------------------------------------------------------------------- */
 
 static int look(Scanner * s) {
-  int state;
+  int state = 0;
   int c = 0;
+  String *str_delimiter = 0;
 
-  state = 0;
   Clear(s->text);
   s->start_line = s->line;
   Setfile(s->text, Getfile(s->str));
+
+
   while (1) {
     switch (state) {
     case 0:
@@ -435,24 +513,30 @@ static int look(Scanner * s) {
 
     case 1000:
       if ((c = nextchar(s)) == 0)
-	return (0);
+        return (0);
       if (c == '%')
 	state = 4;		/* Possibly a SWIG directive */
-
-      /* Look for possible identifiers */
-
+      
+      /* Look for possible identifiers or unicode/delimiter strings */
       else if ((isalpha(c)) || (c == '_') ||
-	       (s->idstart && strchr(s->idstart, c)))
+	       (s->idstart && strchr(s->idstart, c))) {
 	state = 7;
+      }
 
       /* Look for single character symbols */
 
-      else if (c == '(')
+      else if (c == '(') {
+        Scanner_push_brackets(s);
 	return SWIG_TOKEN_LPAREN;
-      else if (c == ')')
+      }
+      else if (c == ')') {
+        Scanner_pop_brackets(s);
 	return SWIG_TOKEN_RPAREN;
-      else if (c == ';')
+      }
+      else if (c == ';') {
+        Scanner_clear_brackets(s);
 	return SWIG_TOKEN_SEMI;
+      }
       else if (c == ',')
 	return SWIG_TOKEN_COMMA;
       else if (c == '*')
@@ -502,16 +586,16 @@ static int look(Scanner * s) {
 	state = 1;		/* Comment (maybe)  */
 	s->start_line = s->line;
       }
-      else if (c == '\"') {
-	state = 2;		/* Possibly a string */
-	s->start_line = s->line;
-	Clear(s->text);
-      }
 
       else if (c == ':')
 	state = 5;		/* maybe double colon */
       else if (c == '0')
 	state = 83;		/* An octal or hex value */
+      else if (c == '\"') {
+	state = 2;              /* A string constant */
+	s->start_line = s->line;
+	Clear(s->text);
+      }
       else if (c == '\'') {
 	s->start_line = s->line;
 	Clear(s->text);
@@ -590,18 +674,63 @@ static int look(Scanner * s) {
       break;
 
     case 2:			/* Processing a string */
+      if (!str_delimiter) {
+	state=20;
+	break;
+      }
+      
       if ((c = nextchar(s)) == 0) {
 	Swig_error(cparse_file, cparse_start_line, "Unterminated string\n");
 	return SWIG_TOKEN_ERROR;
       }
-      if (c == '\"') {
-	Delitem(s->text, DOH_END);
-	return SWIG_TOKEN_STRING;
-      } else if (c == '\\') {
-	Delitem(s->text, DOH_END);
-	get_escape(s);
-      } else
-	state = 2;
+      else if (c == '(') {
+	state = 20;
+      }
+      else {
+	char temp[2] = { 0, 0 };
+	temp[0] = c;
+	Append( str_delimiter, temp );
+      }
+    
+      break;
+
+    case 20:			/* Inside the string */
+      if ((c = nextchar(s)) == 0) {
+	Swig_error(cparse_file, cparse_start_line, "Unterminated string\n");
+	return SWIG_TOKEN_ERROR;
+      }
+      
+      if (!str_delimiter) { /* Ordinary string: "value" */
+	if (c == '\"') {
+	  Delitem(s->text, DOH_END);
+	  return SWIG_TOKEN_STRING;
+	} else if (c == '\\') {
+	  Delitem(s->text, DOH_END);
+	  get_escape(s);
+	}
+      } else {             /* Custom delimiter string: R"XXXX(value)XXXX" */
+	if (c==')') {
+	  int i=0;
+	  String *end_delimiter = NewStringEmpty();
+	  while ((c = nextchar(s)) != 0 && c!='\"') {
+	    char temp[2] = { 0, 0 };
+	    temp[0] = c;
+	    Append( end_delimiter, temp );
+	    i++;
+	  }
+	  
+	  if (Strcmp( str_delimiter, end_delimiter )==0) {
+	    Delete( end_delimiter ); /* Correct end delimiter )XXXX" occured */
+	    Delete( str_delimiter );
+	    str_delimiter = 0;
+	    return SWIG_TOKEN_STRING;
+	  } else {                   /* Incorrect end delimiter occured */
+	    retract( s, i );
+	    Delete( end_delimiter );
+	  }
+	}
+      }
+      
       break;
 
     case 3:			/* Maybe a not equals */
@@ -719,6 +848,7 @@ static int look(Scanner * s) {
       break;
 
     case 60:			/* shift operators */
+      Scanner_inc_brackets(s);
       if ((c = nextchar(s)) == 0)
 	return SWIG_TOKEN_LESSTHAN;
       if (c == '<')
@@ -731,9 +861,10 @@ static int look(Scanner * s) {
       }
       break;
     case 61:
+      Scanner_dec_brackets(s);
       if ((c = nextchar(s)) == 0)
 	return SWIG_TOKEN_GREATERTHAN;
-      if (c == '>')
+      if (c == '>' && ((*Scanner_brackets(s))<0)) /* go to double >> only, if no template < has been used */
 	state = 250;
       else if (c == '=')
 	return SWIG_TOKEN_GTEQUAL;
@@ -742,18 +873,108 @@ static int look(Scanner * s) {
 	return SWIG_TOKEN_GREATERTHAN;
       }
       break;
-    case 7:			/* Identifier */
-      if ((c = nextchar(s)) == 0)
-	state = 71;
-      else if (isalnum(c) || (c == '_') || (c == '$')) {
-	state = 7;
-      } else {
+    
+    case 7:			/* Identifier or true/false or unicode/custom delimiter string */
+      if (c == 'R') { /* Possibly CUSTOM DELIMITER string */
+	state = 72;
+	break;
+      }
+      else if (c == 'L') { /* Probably identifier but may be a wide string literal */
+	state = 77;
+	break;
+      }
+      else if (c != 'u' && c != 'U') { /* Definitely an identifier */
+	state = 70;
+	break;
+      }
+      
+      if ((c = nextchar(s)) == 0) {
+	state = 76;
+      }
+      else if (c == '\"') { /* Definitely u, U or L string */
 	retract(s, 1);
+	state = 1000;
+      }
+      else if (c == 'R') { /* Possibly CUSTOM DELIMITER u, U, L string */
+	state = 73;
+      }
+      else if (c == '8') { /* Possibly u8 string */
 	state = 71;
       }
+      else {
+	retract(s, 1);   /* Definitely an identifier */
+	state = 70;
+      }
       break;
 
-    case 71:			/* Identifier or true/false */
+    case 70:			/* Identifier */
+      if ((c = nextchar(s)) == 0)
+	state = 76;
+      else if (isalnum(c) || (c == '_') || (c == '$')) {
+	state = 70;
+      } else {
+	retract(s, 1);
+	state = 76;
+      }
+      break;
+    
+    case 71:			/* Possibly u8 string */
+      if ((c = nextchar(s)) == 0) {
+	state = 76;
+      }
+      else if (c=='\"') {
+	retract(s, 1); /* Definitely u8 string */
+	state = 1000;
+      }
+      else if (c=='R') {
+	state = 74; /* Possibly CUSTOM DELIMITER u8 string */
+      }
+      else {
+	retract(s, 2); /* Definitely an identifier. Retract 8" */
+	state = 70;
+      }
+      
+      break;
+
+    case 72:			/* Possibly CUSTOM DELIMITER string */
+    case 73:
+    case 74:
+      if ((c = nextchar(s)) == 0) {
+	state = 76;
+      }
+      else if (c=='\"') {
+	retract(s, 1); /* Definitely custom delimiter u, U or L string */
+	str_delimiter = NewStringEmpty();
+	state = 1000;
+      }
+      else {
+	if (state==72) {
+	  retract(s, 1); /* Definitely an identifier. Retract ? */
+	}
+	else if (state==73) {
+	  retract(s, 2); /* Definitely an identifier. Retract R? */
+	}
+	else if (state==74) {
+	  retract(s, 3); /* Definitely an identifier. Retract 8R? */
+	}
+	state = 70;
+      }
+      
+      break;
+
+    case 75:			/* Special identifier $ */
+      if ((c = nextchar(s)) == 0)
+	return SWIG_TOKEN_DOLLAR;
+      if (isalnum(c) || (c == '_') || (c == '*') || (c == '&')) {
+	state = 70;
+      } else {
+	retract(s,1);
+	if (Len(s->text) == 1) return SWIG_TOKEN_DOLLAR;
+	state = 76;
+      }
+      break;
+
+    case 76:			/* Identifier or true/false */
       if (cparse_cplusplus) {
 	if (Strcmp(s->text, "true") == 0)
 	  return SWIG_TOKEN_BOOL;
@@ -763,15 +984,56 @@ static int look(Scanner * s) {
       return SWIG_TOKEN_ID;
       break;
 
-    case 75:			/* Special identifier $ */
+    case 77: /*identifier or wide string literal*/
       if ((c = nextchar(s)) == 0)
-	return SWIG_TOKEN_DOLLAR;
-      if (isalnum(c) || (c == '_') || (c == '*') || (c == '&')) {
+	return SWIG_TOKEN_ID;
+      else if (c == '\"') {
+	s->start_line = s->line;
+	Clear(s->text);
+	state = 78;
+      }
+      else if (c == '\'') {
+	s->start_line = s->line;
+	Clear(s->text);
+	state = 79;
+      }
+      else if (isalnum(c) || (c == '_') || (c == '$'))
 	state = 7;
-      } else {
-	retract(s,1);
-	if (Len(s->text) == 1) return SWIG_TOKEN_DOLLAR;
-	state = 71;
+      else {
+	retract(s, 1);
+	return SWIG_TOKEN_ID;
+      }
+    break;
+
+    case 78:			/* Processing a wide string literal*/
+      if ((c = nextchar(s)) == 0) {
+	Swig_error(cparse_file, cparse_start_line, "Unterminated wide string\n");
+	return SWIG_TOKEN_ERROR;
+      }
+      if (c == '\"') {
+	Delitem(s->text, DOH_END);
+	return SWIG_TOKEN_WSTRING;
+      } else if (c == '\\') {
+	if ((c = nextchar(s)) == 0) {
+	  Swig_error(cparse_file, cparse_start_line, "Unterminated wide string\n");
+	  return SWIG_TOKEN_ERROR;
+	}
+      }
+      break;
+
+    case 79:			/* Processing a wide char literal */
+      if ((c = nextchar(s)) == 0) {
+	Swig_error(cparse_file, cparse_start_line, "Unterminated wide character constant\n");
+	return SWIG_TOKEN_ERROR;
+      }
+      if (c == '\'') {
+	Delitem(s->text, DOH_END);
+	return (SWIG_TOKEN_WCHAR);
+      } else if (c == '\\') {
+	if ((c = nextchar(s)) == 0) {
+	  Swig_error(cparse_file, cparse_start_line, "Unterminated wide character literal\n");
+	  return SWIG_TOKEN_ERROR;
+	}
       }
       break;
 
@@ -1049,6 +1311,7 @@ static int look(Scanner * s) {
       break;
 
     case 240:			/* LSHIFT, LSEQUAL */
+      Scanner_inc_brackets(s);
       if ((c = nextchar(s)) == 0)
 	return SWIG_TOKEN_LSHIFT;
       else if (c == '=')
@@ -1060,6 +1323,7 @@ static int look(Scanner * s) {
       break;
 
     case 250:			/* RSHIFT, RSEQUAL */
+      Scanner_dec_brackets(s);
       if ((c = nextchar(s)) == 0)
 	return SWIG_TOKEN_RSHIFT;
       else if (c == '=')
@@ -1282,6 +1546,114 @@ int Scanner_skip_balanced(Scanner * s, int startchar, int endchar) {
   return 0;
 }
 
+/* -----------------------------------------------------------------------------
+ * Scanner_get_raw_text_balanced()
+ *
+ * Returns raw text between 2 braces, does not change scanner state in any way
+ * ----------------------------------------------------------------------------- */
+
+String *Scanner_get_raw_text_balanced(Scanner *s, int startchar, int endchar) {
+  String *result = 0;
+  char c;
+  int old_line = s->line;
+  String *old_text = Copy(s->text);
+  int position = Tell(s->str);
+
+  int num_levels = 1;
+  int state = 0;
+  char temp[2] = { 0, 0 };
+  temp[0] = (char) startchar;
+  Clear(s->text);
+  Setfile(s->text, Getfile(s->str));
+  Setline(s->text, s->line);
+  Append(s->text, temp);
+  while (num_levels > 0) {
+    if ((c = nextchar(s)) == 0) {
+      Clear(s->text);
+      Append(s->text, old_text);
+      Delete(old_text);
+      s->line = old_line;
+      return 0;
+    }
+    switch (state) {
+    case 0:
+      if (c == startchar)
+	num_levels++;
+      else if (c == endchar)
+	num_levels--;
+      else if (c == '/')
+	state = 10;
+      else if (c == '\"')
+	state = 20;
+      else if (c == '\'')
+	state = 30;
+      break;
+    case 10:
+      if (c == '/')
+	state = 11;
+      else if (c == '*')
+	state = 12;
+      else if (c == startchar) {
+	state = 0;
+	num_levels++;
+      }
+      else
+	state = 0;
+      break;
+    case 11:
+      if (c == '\n')
+	state = 0;
+      else
+	state = 11;
+      break;
+    case 12: /* first character inside C comment */
+      if (c == '*')
+	state = 14;
+      else
+	state = 13;
+      break;
+    case 13:
+      if (c == '*')
+	state = 14;
+      break;
+    case 14: /* possible end of C comment */
+      if (c == '*')
+	state = 14;
+      else if (c == '/')
+	state = 0;
+      else
+	state = 13;
+      break;
+    case 20:
+      if (c == '\"')
+	state = 0;
+      else if (c == '\\')
+	state = 21;
+      break;
+    case 21:
+      state = 20;
+      break;
+    case 30:
+      if (c == '\'')
+	state = 0;
+      else if (c == '\\')
+	state = 31;
+      break;
+    case 31:
+      state = 30;
+      break;
+    default:
+      break;
+    }
+  }
+  Seek(s->str, position, SEEK_SET);
+  result = Copy(s->text);
+  Clear(s->text);
+  Append(s->text, old_text);
+  Delete(old_text);
+  s->line = old_line;
+  return result;
+}
 /* -----------------------------------------------------------------------------
  * Scanner_isoperator()
  *
